@@ -1,6 +1,7 @@
 from importlib import resources
 import json
 from pymongo import MongoClient
+from pymongo.collection import Collection
 from pymatgen.core import Composition
 
 import numpy as np
@@ -20,6 +21,7 @@ import xlsxwriter
 from io import BytesIO
 from typing import List, Dict, Tuple, Union
 
+
 class Analyzer:
     '''Base class for all analyzers. Initializes a connection to the database and collection. Also contains some helper
     functions for data analysis, such as getting a list of all unique DOIs in the collection.
@@ -27,42 +29,86 @@ class Analyzer:
     Args:
         database: Name of the database to connect to.
         collection: Name of the collection to connect to.
+        collectionManualOverride: If specified, the collectionManualOverride is used instead of the database and
+            collection arguments. It expects a pymongo.collection.Collection object, however, it is not type-checked
+            (only hinted) to allow for more flexibility, including instances of
+            [MontyDB](https://github.com/davidlatwe/MontyDB) Collection class or
+            [Mongomock](https://github.com/mongomock/mongomock) Collection class. Defaults to None and has no effect
+            in that case.
 
     Note:
         The credentials for the database are stored in the credentials.json file in the pyqalloy package. This access
         credentials are not included in the public repository.
     '''
-    def __init__(self, database: str, collection: str):
-        with resources.files('pyqalloy').joinpath('credentials.json').open('r') as f:
-            self.credentials = json.load(f)
-        self.ultera_database_uri = f"mongodb+srv://{self.credentials['name']}:{self.credentials['dbKey']}" \
-                                   f"@{self.credentials['dataServer']}"
-        self.ultera_client = MongoClient(self.ultera_database_uri)
-        self.collection = self.ultera_client[database][collection]
-        print(f'Connected to the {collection} in {database} with {self.collection.estimated_document_count()} data '
-              f'points detected.')
+
+    def __init__(self,
+                 database: str,
+                 collection: str,
+                 collectionManualOverride: Collection = None):
+        if collectionManualOverride is not None:
+            self.collectionManualOverrideSet = True
+            self.collection = collectionManualOverride
+            self.ultera_database_uri = None
+            self.ultera_client = None
+        else:
+            self.collectionManualOverrideSet = False
+            with resources.files('pyqalloy').joinpath('credentials.json').open('r') as f:
+                self.credentials = json.load(f)
+            self.ultera_database_uri = f"mongodb+srv://{self.credentials['name']}:{self.credentials['dbKey']}" \
+                                       f"@{self.credentials['dataServer']}"
+            self.ultera_client = MongoClient(self.ultera_database_uri)
+            self.collection = self.ultera_client[database][collection]
+            print(f'Connected to the {collection} in {database} with {self.collection.estimated_document_count()} data '
+                  f'points detected.')
 
     def get_allDOIs(self) -> List[str]:
-        '''Returns a list of all unique DOIs in the collection. This is useful for iterating over all publications in the
-        collection.'''
-        return [e['doi'] for e in self.collection.aggregate([
-            {'$match': {'reference.doi': {'$ne': None}}},
-            {'$group': {'_id': '$reference.doi'}},
-            {'$set': {'doi': '$_id', '_id': '$$REMOVE'}}])]
+        '''Returns a list of all unique DOIs in the collection. This is useful for iterating over all publications in
+        the collection. If the `collectionManualOverride` is left as None, the function uses the MongoDB aggregation
+        pipeline to perform the operation efficiently on the server side. If the `collectionManualOverride` is specified,
+        the find method is used instead, which is less efficient, but works with other database objects, such as
+        [MontyDB](https://github.com/davidlatwe/MontyDB).
+        '''
+        if not self.collectionManualOverrideSet:
+            # Leveraging MongoDB aggregation pipeline to get a list of all unique DOIs efficiently on the server side
+            return [e['doi'] for e in self.collection.aggregate([
+                {'$match': {'reference.doi': {'$ne': None}}},
+                {'$group': {'_id': '$reference.doi'}},
+                {'$set': {'doi': '$_id', '_id': '$$REMOVE'}}])]
+        else:
+            # In case of a manual override, user is usually trying to "mock" the database and collection objects so
+            # the aggregation pipeline may not be available. In that case, we have to utilize the find() method and
+            # iterate over all documents in the collection.
+            allDOIs = set()
+            for e in self.collection.find({'reference.doi': {'$ne': None}}, {'reference.doi': 1}):
+                allDOIs.add(e['reference']['doi'])
+            return list(allDOIs)
+
 
 class SingleDOIAnalyzer(Analyzer):
-    '''Extends the Analyzer class. It is used to asses the data coming from a single publication based on the DOI string.
+    '''Extends the Analyzer class. It is used to assess the data coming from a single publication based on the DOI string.
 
     Args:
         doi: DOI string of the publication to analyze. Defaults to None.
-        name: Name of the researcher who uploaded the data. Allows liimiting the analysis to what a person was responsible
-            for. Defaults to None.
+        name: Name of the researcher who uploaded the data. This setting allows limiting the analysis to a person who
+            was responsible for the upload. Defaults to None.
         database: Name of the database to connect to. Defaults to 'ULTERA_internal'.
         collection: Name of the collection to connect to. Defaults to 'CURATED_Dec2022'.
+        collectionManualOverride: If specified, the collectionManualOverride is used instead of the database and
+            collection arguments. It expects a pymongo.collection.Collection object, however, it is not type-checked
+            (only hinted) to allow for more flexibility, including instances of
+            [MontyDB](https://github.com/davidlatwe/MontyDB) Collection class or
+            [Mongomock](https://github.com/mongomock/mongomock) Collection class. Defaults to None and has no effect
+            in that case.
 
     '''
-    def __init__(self, doi=None, name=None, database='ULTERA_internal', collection='CURATED_Dec2022'):
-        super().__init__(database=database, collection=collection)
+
+    def __init__(self,
+                 doi: str = None,
+                 name: str = None,
+                 database: str = 'ULTERA_internal',
+                 collection: str = 'CURATED_Dec2022',
+                 collectionManualOverride: Collection = None):
+        super().__init__(database=database, collection=collection, collectionManualOverride=collectionManualOverride)
         self.name = name
         self.doi = doi
         self.resetVariables()
@@ -112,10 +158,10 @@ class SingleDOIAnalyzer(Analyzer):
                 self.names.add(e['meta']['name'])
                 self.els.update(list(c.get_el_amt_dict().keys()))
                 self.fStrings.append(
-                    e['material']['percentileFormula']#+'<br>'+
-                    #e['material']['percentileFormula']+'<br>'+
-                    #e['material']['relationalFormula']
-                    )
+                    e['material']['percentileFormula']  # +'<br>'+
+                    # e['material']['percentileFormula']+'<br>'+
+                    # e['material']['relationalFormula']
+                )
             if 'pointer' in e['reference']:
                 self.pointers.add(e['reference']['pointer'])
         # Vectorize based on a list of elements
@@ -133,7 +179,7 @@ class SingleDOIAnalyzer(Analyzer):
         self.getCompVecs()
         self.nn_distances = [l[1] for l in nn.fit(self.compVecs).kneighbors(self.compVecs)[0]]
 
-    def print_nnDistances(self, minSamples: int=2, printOut: bool=True) -> None:
+    def print_nnDistances(self, minSamples: int = 2, printOut: bool = True) -> None:
         '''Prints the nearest neighbor distances for all unique composition vectors in the publication. The distances
         are calculated using the L1 metric and the k-d tree algorithm. The distances are normalized to the maximum
         distance in the publication. The output is persisted in the self.printLog variable.
@@ -143,16 +189,16 @@ class SingleDOIAnalyzer(Analyzer):
             printOut: If True, the results are printed to the console. Defaults to True.
 
         '''
-        assert len(self.compVecs)>0
-        assert len(self.nn_distances)>0
-        assert len(self.formulas)==len(self.nn_distances)
-        if len(self.nn_distances)>=minSamples:
+        assert len(self.compVecs) > 0
+        assert len(self.nn_distances) > 0
+        assert len(self.formulas) == len(self.nn_distances)
+        if len(self.nn_distances) >= minSamples:
             if self.name is None or self.name in self.names:
                 maxD = max(self.nn_distances)
                 self.printLog += f'\n--->  {self.doi}'
                 for l, f in zip(self.nn_distances, self.formulas):
-                    temp_line = f'{round(l, 4):<10}|  {round(l/maxD, 4):<10} <-- {f}'
-                    self.printLog += temp_line+'\n'
+                    temp_line = f'{round(l, 4):<10}|  {round(l / maxD, 4):<10} <-- {f}'
+                    self.printLog += temp_line + '\n'
                     if printOut:
                         print(temp_line)
                 self.printLog += '\n'
@@ -175,7 +221,7 @@ class SingleDOIAnalyzer(Analyzer):
             List of 2D PCA coordinates for all composition vectors.
 
         '''
-        assert len(self.compVecs)>0
+        assert len(self.compVecs) > 0
         pca = PCA(n_components=2)
         self.compVecs_2DPCA = pca.fit_transform(self.compVecs)
         self.compVecs_2DPCA_minRangeInDim = min([
@@ -184,7 +230,7 @@ class SingleDOIAnalyzer(Analyzer):
 
         return self.compVecs_2DPCA
 
-    def analyze_compVecs_2DPCA(self, minDistance: float=0.001, showFigure: bool=True) -> Union[str, BytesIO]:
+    def analyze_compVecs_2DPCA(self, minDistance: float = 0.001, showFigure: bool = True) -> Union[str, BytesIO]:
         '''Performs a 2D PCA on the composition vectors. The results are stored in the self.compVecs_2DPCA variable.
         The minimum range in both dimensions is stored in the self.compVecs_2DPCA_minRangeInDim variable.
         The results are plotted using plotly. The figure is stored in the self.fig variable.
@@ -237,8 +283,9 @@ class SingleDOIAnalyzer(Analyzer):
         assert isinstance(self.compVecs_2DPCA_plot, BytesIO)
         workbook = xlsxwriter.Workbook(workbookPath)
         worksheet = workbook.add_worksheet()
-        cellIndex = f'A{1+skipLines}'
-        worksheet.insert_image(cellIndex, self.doi, {'image_data': self.compVecs_2DPCA_plot, 'x_scale': 0.2, 'y_scale': 0.2})
+        cellIndex = f'A{1 + skipLines}'
+        worksheet.insert_image(cellIndex, self.doi,
+                               {'image_data': self.compVecs_2DPCA_plot, 'x_scale': 0.2, 'y_scale': 0.2})
         workbook.close()
 
     def writeManyPlots(self, toPlotList: list, workbookPath: str) -> None:
@@ -274,21 +321,31 @@ class SingleCompositionAnalyzer(Analyzer):
         name: Name of the researcher to limit the search to. Defaults to None.
         database: Name of the database to use. Defaults to 'ULTERA_internal'.
         collection: Name of the collection to use. Defaults to 'CURATED_Dec2022'.
+        collectionManualOverride: If specified, the collectionManualOverride is used instead of the database and
+            collection arguments. It expects a pymongo.collection.Collection object, however, it is not type-checked
+            (only hinted) to allow for more flexibility, including instances of
+            [MontyDB](https://github.com/davidlatwe/MontyDB) Collection class or
+            [Mongomock](https://github.com/mongomock/mongomock) Collection class. Defaults to None and has no effect
+            in that case.
     '''
 
-    def __init__(self, name: str=None, database: str='ULTERA_internal', collection: str='CURATED_Dec2022'):
-        super().__init__(database=database, collection=collection)
+    def __init__(self,
+                 name: str = None,
+                 database: str = 'ULTERA_internal',
+                 collection: str = 'CURATED_Dec2022',
+                 collectionManualOverride: Collection = None):
+        super().__init__(database=database, collection=collection, collectionManualOverride=collectionManualOverride)
         self.name = name
         self.formulas = set()
         self.printOuts = list()
 
     def scanCompositionsAround100(self,
-                                  lowerBound: float=80,
-                                  uncertainty: float=0.21,
-                                  upperBound: float=120,
-                                  queryLimit: int=10000,
-                                  resultLimit: int=1000,
-                                  printOnFly: bool=False) -> None:
+                                  lowerBound: float = 80,
+                                  uncertainty: float = 0.21,
+                                  upperBound: float = 120,
+                                  queryLimit: int = 10000,
+                                  resultLimit: int = 1000,
+                                  printOnFly: bool = False) -> None:
         '''Scans the database for compositions around 100% but not exactly 100% as defined by the lower and upper bounds.
         Results are stored in self.printOuts and can be printed out or written to a file using self.writeResultsToFile().
 
@@ -324,7 +381,7 @@ class SingleCompositionAnalyzer(Analyzer):
 
                 def printAlloy(self):
                     '''Prints the alloy to the console while retaining the information in self.printOuts list'''
-                    printOut =  f"DOI: {e['reference']['doi']}"
+                    printOut = f"DOI: {e['reference']['doi']}"
                     if 'pointer' in e['reference']:
                         printOut += f"  --> {e['reference']['pointer']}"
                     printOut += f"\nF:   {f}\n"
@@ -337,24 +394,22 @@ class SingleCompositionAnalyzer(Analyzer):
                     if printOnFly:
                         print(printOut)
 
-                if fracsSum>lowerBound and fracsSum<(100-uncertainty):
+                if fracsSum > lowerBound and fracsSum < (100 - uncertainty):
                     printAlloy(self)
-                elif fracsSum>lowerBound/100 and fracsSum<(100-uncertainty)/100:
+                elif fracsSum > lowerBound / 100 and fracsSum < (100 - uncertainty) / 100:
                     printAlloy(self)
-                elif fracsSum<upperBound and fracsSum>(100+uncertainty):
+                elif fracsSum < upperBound and fracsSum > (100 + uncertainty):
                     printAlloy(self)
-                elif fracsSum<upperBound/100 and fracsSum>(100+uncertainty)/100:
+                elif fracsSum < upperBound / 100 and fracsSum > (100 + uncertainty) / 100:
                     printAlloy(self)
 
-
-
     def writeResultsToFile(self, fileName: str) -> None:
         '''Writes the results to a file. The file is created if it does not exist, otherwise it is overwritten.
 
         Args:
             fileName: Name of the file to write the results to.
         '''
-        assert len(self.printOuts)>0
+        assert len(self.printOuts) > 0
         with open(fileName, 'w+') as f:
             f.write(datetime.now().strftime("%c"))
             f.write('\n')
@@ -362,6 +417,7 @@ class SingleCompositionAnalyzer(Analyzer):
                 f.write(printOut)
                 f.write('\n')
 
+
 class AllDataAnalyzer(Analyzer):
     '''Class to analyze datapoints in the scope of the contents of entire database. It primarily relies on clustering
     analysis to identify outliers and anomalies in a few different ways.
@@ -370,6 +426,12 @@ class AllDataAnalyzer(Analyzer):
         database: Name of the database to use. Defaults to 'ULTERA_internal'.
         collection: Name of the collection to use. Defaults to 'CURATED_Dec2022'.
         name: Name of the researcher to limit the search to. Defaults to None.
+        collectionManualOverride: If specified, the collectionManualOverride is used instead of the database and
+            collection arguments. It expects a pymongo.collection.Collection object, however, it is not type-checked
+            (only hinted) to allow for more flexibility, including instances of
+            [MontyDB](https://github.com/davidlatwe/MontyDB) Collection class or
+            [Mongomock](https://github.com/mongomock/mongomock) Collection class. Defaults to None and has no effect
+            in that case.
 
     Properties:
         allComps: List of all unique compositions in the database. It is automatically updated when the class is
@@ -379,20 +441,23 @@ class AllDataAnalyzer(Analyzer):
         outliers: List of outliers in the database identified by the last used method (e.g. DBSCAN).
     '''
 
-    def __init__(self, database: str='ULTERA_internal', collection: str='CURATED_Dec2022', name: str=None):
-        super().__init__(database=database, collection=collection)
+    def __init__(self,
+                 database: str = 'ULTERA_internal',
+                 collection: str = 'CURATED_Dec2022',
+                 name: str = None,
+                 collectionManualOverride: Collection = None):
+        super().__init__(database=database, collection=collection, collectionManualOverride=collectionManualOverride)
         self.name = name
         self.outliers = list()
         self.els = set()
 
         self.allComps = self.updateAllComps(printOut=False, printOutMinimal=True)
 
-
-    def updateAllComps(self, printOut: bool=False, printOutMinimal: bool=True) -> list:
-        '''Idenitfies a list of all unique compositions in the database, updates the self.els property, and then converts
+    def updateAllComps(self, printOut: bool = False, printOutMinimal: bool = True) -> list:
+        '''Identifies a list of all unique compositions in the database, updates the self.els property, and then converts
         the list of compositions into a list of dictionaries with the formula and a vector representation of the composition
         in the order of self.els. The vector representation is used for full-dimensional clustering analysis. Some other methods
-        like TSNE enbedding will update these dictionaries with additional keys.
+        like TSNE embedding will update these dictionaries with additional keys.
 
         Args:
             printOut: If True, prints out the list of all unique compositions. Defaults to False.
@@ -406,9 +471,8 @@ class AllDataAnalyzer(Analyzer):
         print('Updating the list of all unique composition points...')
         formulas = set()
         for e in self.collection.find({
-                'material.nComponents': {'$gte': 3},
-                'reference.doi': {'$ne': None}}):
-
+            'material.nComponents': {'$gte': 3},
+            'reference.doi': {'$ne': None}}):
             rf = e['material']['relationalFormula']
             c = Composition(rf)
             formulas.add(rf)
@@ -435,14 +499,14 @@ class AllDataAnalyzer(Analyzer):
 
         return comps
 
-    def getTSNE(self, perplexity: int=2, init: str='pca') -> np.ndarray:
+    def getTSNE(self, perplexity: int = 2, init: str = 'pca') -> np.ndarray:
         '''Performs TSNE embedding on the list of compositions in self.allComps. The TSNE embedding is stored in the
         'compVec_TSNE2D' key of each dictionary in self.allComps. The TSNE embedding is also returned as a numpy array.
 
         Args:
             perplexity: Perplexity parameter for the TSNE embedding. Defaults to 2. This is the parameter that controls
                 the number of alloys that are expected to be close to each other in the embedding. The value of 2 is
-                chosen for visualizing outlier detection becuase the database is very sparse, populated by chains of
+                chosen for visualizing outlier detection because the database is very sparse, populated by chains of
                 neighboring alloys, and we do expect many without more than one neighbor. For more general use, the
                 default value of 5-10 is recommended. The value of 30, often used in the literature, is not recommended
                 for HEA datasets.
@@ -468,10 +532,10 @@ class AllDataAnalyzer(Analyzer):
         Returns:
             None
         '''
-        assert len(self.allComps)>0
+        assert len(self.allComps) > 0
         assert 'formula' in self.allComps[0]
         assert 'compVec_TSNE2D' in self.allComps[0]
-        assert len(self.allComps[0]['compVec_TSNE2D'])==2
+        assert len(self.allComps[0]['compVec_TSNE2D']) == 2
 
         fig = px.scatter(x=[c['compVec_TSNE2D'][0] for c in self.allComps],
                          y=[c['compVec_TSNE2D'][1] for c in self.allComps],
@@ -484,7 +548,7 @@ class AllDataAnalyzer(Analyzer):
                          )
         fig.show()
 
-    def getDBSCAN(self, eps: float=0.3, min_samples: int=2, p: int=1) -> Tuple[np.ndarray, int]:
+    def getDBSCAN(self, eps: float = 0.3, min_samples: int = 2, p: int = 1) -> Tuple[np.ndarray, int]:
         '''Performs DBSCAN clustering on the list of compositions in self.allComps. The DBSCAN clustering is stored in the
         'dbscanCluster' key of each dictionary in self.allComps. The DBSCAN clustering is also returned as a numpy array
         along with the number of outliers identified.
@@ -506,12 +570,12 @@ class AllDataAnalyzer(Analyzer):
             Numpy array of the DBSCAN clustering and the number of outliers identified.
         '''
 
-        assert len(self.allComps)>0
+        assert len(self.allComps) > 0
         assert 'compVec' in self.allComps[0]
 
         dbscan = DBSCAN(eps=eps, min_samples=min_samples, p=p)
         X = np.array([c['compVec'] for c in self.allComps])
-        dbscanClusters =  dbscan.fit_predict(X)
+        dbscanClusters = dbscan.fit_predict(X)
 
         outlierN = 0
         for i, c in enumerate(self.allComps):
@@ -520,11 +584,11 @@ class AllDataAnalyzer(Analyzer):
                 outlierN += 1
 
         print(f'Found {len(set(dbscanClusters))} clusters and {outlierN} outliers.')
-        print(f'Outlier ratio: {round(outlierN/len(dbscanClusters)*100,1)}%')
+        print(f'Outlier ratio: {round(outlierN / len(dbscanClusters) * 100, 1)}%')
 
         return dbscanClusters, outlierN
 
-    def getDBSCANautoEpsilon(self, outlierTargetN: int=10) -> Tuple[np.ndarray, int]:
+    def getDBSCANautoEpsilon(self, outlierTargetN: int = 10) -> Tuple[np.ndarray, int]:
         '''Performs DBSCAN clustering using getDBSCAN() with a range of epsilon values until the desired minimum number
         of outliers is found. It efficiently allows user to find as many outliers as they can investigate independently
         of the number of alloys in the dataset. The DBSCAN clustering is stored in the 'dbscanCluster' key of each dictionary in
@@ -537,18 +601,18 @@ class AllDataAnalyzer(Analyzer):
         Returns:
             Numpy array of the DBSCAN clustering and the number of outliers identified.
         '''
-        assert len(self.allComps)>outlierTargetN
+        assert len(self.allComps) > outlierTargetN
         outlierN = 0
         eps = 1.00001
         assert outlierTargetN > 0
+        assert outlierN < outlierTargetN
         while outlierN < outlierTargetN:
-            print(f'Running DBSCAN with eps={round(eps,3)}...')
+            print(f'Running DBSCAN with eps={round(eps, 3)}...')
             dbscanClusters, outlierN = self.getDBSCAN(eps=eps)
             eps -= 0.025
 
         return dbscanClusters, outlierN
 
-
     def showClustersDBSCAN(self):
         '''Plots the TSNE embedding of the compositions in self.allComps colored by the DBSCAN clustering. The plot is
         interactive and allows for hovering over the points to see the formula of the alloy as well as the DBSCAN
@@ -557,7 +621,7 @@ class AllDataAnalyzer(Analyzer):
         Returns:
             None
         '''
-        assert len(self.allComps)>0
+        assert len(self.allComps) > 0
         assert 'formula' in self.allComps[0]
         assert 'dbscanCluster' in self.allComps[0]
         assert 'compVec_TSNE2D' in self.allComps[0]
@@ -595,7 +659,7 @@ class AllDataAnalyzer(Analyzer):
         Returns:
             None
         '''
-        assert len(self.allComps)>0
+        assert len(self.allComps) > 0
         assert 'formula' in self.allComps[0]
         assert 'dbscanCluster' in self.allComps[0]
         assert 'compVec_TSNE2D' in self.allComps[0]
@@ -603,7 +667,7 @@ class AllDataAnalyzer(Analyzer):
 
         fig = px.scatter(x=[c['compVec_TSNE2D'][0] for c in self.allComps],
                          y=[c['compVec_TSNE2D'][1] for c in self.allComps],
-                         color=['outlier' if c['dbscanCluster']==-1 else 'clustered' for c in self.allComps],
+                         color=['outlier' if c['dbscanCluster'] == -1 else 'clustered' for c in self.allComps],
                          hover_name=[c['formula'] for c in self.allComps],
                          color_discrete_sequence=px.colors.qualitative.Dark24,
                          labels={'x': f'{len(self.els)}D->2D TSNE1',
@@ -613,7 +677,7 @@ class AllDataAnalyzer(Analyzer):
                          )
         fig.show()
 
-    def findOutlierDataSources(self, filterByName: bool=False) -> list:
+    def findOutlierDataSources(self, filterByName: bool = False) -> list:
         '''Finds the data sources for the outliers identified by DBSCAN. If filterByName is True, only data sources
         with the same name as the current analyzer name setting will be printed. Otherwise, all data sources will be
         printed.
@@ -625,10 +689,11 @@ class AllDataAnalyzer(Analyzer):
         Returns:
             List of dictionaries containing the data sources for the outliers.
         '''
-        assert len(self.outliers)>0
+        assert len(self.outliers) > 0
         assert 'formula' in self.outliers[0]
 
         outlierSources = list()
+
         def printEntry(entry):
             out = f'Outlier {outlier["formula"]:<25} | {entry["material"]["percentileFormula"]:<25} | {entry["material"]["rawFormula"]}\n'
             out += f'matched to:  {entry["meta"]["name"]:<20} upload '
@@ -636,7 +701,7 @@ class AllDataAnalyzer(Analyzer):
                 out += f'from DOI {entry["reference"]["doi"]}'
             if 'pointer' in entry['reference']:
                 out += f' at position {entry["reference"]["pointer"]}'
-            print(out,'\n')
+            print(out, '\n')
 
         for outlier in self.outliers:
             e = self.collection.find_one({'material.relationalFormula': outlier['formula']})
